"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Listr = require('listr');
const utils_1 = require("../utils");
const tasks_1 = require("../tasks");
const chalk = require('chalk');
// todo: 'npm-link` doesn't track adding new files,
// so watch mode should be added
function npmLinkCommand({ project, local, deep, verbose, here, debug }) {
    const noDeepLinking = deep === false;
    // 1. clean dist folders
    // 2.1 merge pkg json
    // 2.2 validate pkg (main, module, types)
    // 2.3 write pkg
    // 3. compile ts
    return utils_1.findSubmodules(project, { local })
        .then((opts) => new Listr([
        {
            title: 'Task: ' + chalk.blue('Link all sub modules'),
            task: () => {
                const linkingTasks = new Listr(opts.map(opt => ({
                    title: `npm link` + chalk.cyan(` ${opt.pkg.name}`) + chalk.cyan(` (from: ${opt.dist})`),
                    task: () => tasks_1.npmLink({ cwd: opt.dist, debug })
                })));
                if (noDeepLinking) {
                    return linkingTasks;
                }
                opts.filter(opt => opt.cross.length > 0)
                    .forEach(opt => opt.cross
                    .forEach(crossName => linkingTasks.add({
                    title: `npm link ` + chalk.cyan(`${crossName}`) + ` to ` + chalk.cyan(`${opt.pkg.name} (${opt.src})`),
                    task: () => tasks_1.npmLink({ cwd: opt.dist, module: crossName, debug })
                })));
                return linkingTasks;
            }
        },
        {
            title: 'Task: ' + chalk.blue('Link submodules to local project'),
            task: () => new Listr(opts.map(opt => ({
                title: `npm link ` + chalk.cyan(`${opt.pkg.name}`),
                task: () => tasks_1.npmLink({ module: opt.pkg.name, cwd: '.', debug })
            }))),
            skip: () => here !== true
        }
    ], { renderer: verbose ? 'verbose' : 'default' }));
}
exports.npmLinkCommand = npmLinkCommand;
function run(cli) {
    const { project, verbose, local, deep, here, debug } = cli.flags;
    if (cli.flags.debug) {
        console.log(chalk.bgMagenta('[NPM-SM] DEBUG:'), chalk.magenta('Project Src [' + project + ']'));
        console.log(chalk.bgMagenta('[NPM-SM] DEBUG:'), chalk.magenta('Use Local [' + local + ']'));
        console.log(chalk.bgMagenta('[NPM-SM] DEBUG:'), chalk.magenta('Deep Link [' + deep + ']'));
        console.log(chalk.bgMagenta('[NPM-SM] DEBUG:'), chalk.magenta('To Here [' + here + ']'));
        console.log(chalk.bgMagenta('[NPM-SM] DEBUG:'), chalk.magenta('Debug [' + debug + ']'));
    }
    return npmLinkCommand({ project, verbose, local, deep, here, debug })
        .then(tasks => tasks.run());
}
exports.run = run;
