"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// import { TsmOptions } from '../types';
const utils_1 = require("../utils");
const tasks_1 = require("../tasks");
const link_command_1 = require("./link.command");
/**
 * Heavily inspired by https://github.com/sindresorhus/np
 * */
const execa = require('execa');
const Listr = require("listr");
const chalk = require('chalk');
function run(cli, { buildCommand }) {
    const { project, verbose, tag, access, anyBranch, skipCleanup, skipGitCheck, yolo, skipPublish, debug } = cli.flags;
    if (cli.flags.debug) {
        console.log(chalk.bgMagenta('[NPM-SM] DEBUG:'), chalk.magenta('Project Src [' + project + ']'));
        console.log(chalk.bgMagenta('[NPM-SM] DEBUG:'), chalk.magenta('Project Tag [' + tag + ']'));
        console.log(chalk.bgMagenta('[NPM-SM] DEBUG:'), chalk.magenta('Access [' + access + ']'));
        console.log(chalk.bgMagenta('[NPM-SM] DEBUG:'), chalk.magenta('Debug [' + debug + ']'));
        console.log(chalk.bgMagenta('[NPM-SM] DEBUG:'), chalk.magenta('Any Branch [' + anyBranch + ']'));
        console.log(chalk.bgMagenta('[NPM-SM] DEBUG:'), chalk.magenta('Skip Clean [' + skipCleanup + ']'));
        console.log(chalk.bgMagenta('[NPM-SM] DEBUG:'), chalk.magenta('Skip Git Checks [' + skipGitCheck + ']'));
        console.log(chalk.bgMagenta('[NPM-SM] DEBUG:'), chalk.magenta('Quick Run [' + yolo + ']'));
        console.log(chalk.bgMagenta('[NPM-SM] DEBUG:'), chalk.magenta('Skip Publish [' + skipPublish + ']'));
    }
    return utils_1.findSubmodules(project)
        .then((opts) => {
        const tasks = new Listr([
            {
                title: 'Task: ' + chalk.blue('Git checks'),
                task: () => tasks_1.gitPreCheck({ anyBranch }),
                skip: () => skipGitCheck || yolo
            },
            // test command
            {
                title: 'Task: ' + chalk.blue('Install dependencies'),
                task: () => new Listr(tasks_1.npmInstall({ skipCleanup })),
                skip: () => yolo
            },
            {
                title: 'Task: ' + chalk.blue('Run unit tests'),
                task: () => execa('npm', ['test']),
                skip: () => yolo
            },
            // e2e command
            {
                title: 'Task: ' + chalk.blue('Build sub modules for e2e'),
                task: () => buildCommand({ project, verbose, clean: true, local: true }),
                skip: () => yolo
            },
            {
                title: 'Task: ' + chalk.blue('Link sub modules'),
                task: () => link_command_1.npmLinkCommand({ project, local: true, deep: true, verbose, here: true, debug }),
                skip: () => yolo
            },
            // publish
            // set numeric package version before publish
            {
                title: 'Task: ' + chalk.blue('Build sub modules for publish'),
                task: () => buildCommand({ project, verbose, clean: true, local: false })
            },
            {
                title: 'Task: ' + chalk.blue('Publish all sub modules'),
                task: () => new Listr(opts.map(opt => ({
                    title: `npm publish` + chalk.cyan(` ${opt.pkg.name}`) + chalk.cyan(` (from: ${opt.dist})`),
                    task: () => tasks_1.npmPublish({ cwd: opt.dist, tag, access })
                }))),
                skip: () => skipPublish
            },
            {
                title: 'Task: ' + chalk.blue('Push tags'),
                task: () => execa('git', ['push', '--follow-tags']),
                skip: () => true
            }
        ], { renderer: verbose ? 'verbose' : 'default' });
        return tasks.run();
    });
}
exports.run = run;
