"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// import { TsmOptions } from '../types';
const execa = require('execa');
const path = require('path');
const Listr = require("listr");
const submodules_resolution_1 = require("../utils/submodules-resolution");
const npmVersion_task_1 = require("../tasks/npm/npmVersion.task");
const chalk = require('chalk');
function run(cli) {
    const { project, verbose, message, gitTagVersion, skipPush } = cli.flags;
    const noGitTagVersion = gitTagVersion === false;
    const version = cli.input[1];
    if (cli.flags.debug) {
        console.log(chalk.bgMagenta('[NPM-SM] DEBUG:'), chalk.magenta('Project Src [' + project + ']'));
        console.log(chalk.bgMagenta('[NPM-SM] DEBUG:'), chalk.magenta('Version Message [' + message + ']'));
        console.log(chalk.bgMagenta('[NPM-SM] DEBUG:'), chalk.magenta('git Tag Version [' + gitTagVersion + ']'));
        console.log(chalk.bgMagenta('[NPM-SM] DEBUG:'), chalk.magenta('Skip Push [' + skipPush + ']'));
        console.log(chalk.bgMagenta('[NPM-SM] DEBUG:'), chalk.magenta('Version [' + version + ']'));
    }
    if (!version) {
        return Promise.reject('Error: please provide version like (patch, major, prerelase, 1.2.3, etc.');
    }
    return submodules_resolution_1.findSubmodules(project)
        .then((opts) => {
        // 1. version all sub modules
        // 2. version root package
        const tasks = new Listr([
            {
                title: 'Task: ' + chalk.blue('Version all sub modules'),
                task: () => new Listr(opts.map(opt => ({
                    title: `npm version` + chalk.cyan(`(${opt.pkg.name}: ${opt.src})`),
                    task: () => npmVersion_task_1.npmVersion({ src: opt.src, version, noGitTagVersion: true })
                })))
            },
            {
                title: 'Task: ' + chalk.blue('git add all files'),
                task: () => execa.shell('git add -A'),
                skip: () => noGitTagVersion
            },
            {
                title: 'Task: ' + chalk.blue('Version root package'),
                task: () => npmVersion_task_1.npmVersion({ src: '.', version, message, noGitTagVersion: true })
            },
            {
                title: 'Task: ' + chalk.blue('Create version commit'),
                task: () => {
                    const pkg = require(path.resolve('package.json'));
                    return execa.stdout('git', ['commit', '-am', message || `v${pkg.version}`]);
                },
                skip: () => noGitTagVersion
            },
            {
                title: 'Task: ' + chalk.blue('Add tag version'),
                task: () => {
                    const pkg = require(path.resolve('package.json'));
                    return execa.stdout('git', ['tag', `v${pkg.version}`]);
                },
                skip: () => noGitTagVersion
            },
            {
                title: 'Task: ' + chalk.blue('Push to origin with tags'),
                task: () => {
                    return execa.stdout('git', ['push', '--tags']);
                },
                skip: () => skipPush
            }
        ], { renderer: verbose ? 'verbose' : 'default' });
        return tasks.run();
    });
}
exports.run = run;
