"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const constants_1 = require("../../utils/constants");
const merge_package_json_1 = require("../../utils/merge-package-json");
// import { TsmOptions } from '../../types';
// todo: replace
const readPkg = require('read-pkg');
const writePkg = require('write-pkg');
/**
 * Smart merge of package.json files like Object.assign({}, src, dist)
 * @param opts
 * @param localDependencies
 * @param options
 * @returns {any}
 */
function buildPkgJson(opts, localDependencies, options) {
    // read base package.json
    const base = readPkg.sync(constants_1.ROOT);
    // read package.json in module root folder
    const module = readPkg.sync(opts.src);
    // merge packages
    const pkg = merge_package_json_1.mergePackageJson({ base, module, localDependencies });
    // write packages
    // todo: for some reason, read pkg ignores readme.md and says that readme not found, and this is not true
    delete pkg.readme;
    return writePkg(opts.dist, pkg);
}
exports.buildPkgJson = buildPkgJson;
/**
 * Build our package dependency lists
 * @param tsmOptions
 * @param options
 */
function buildPkgs(tsmOptions, options) {
    // 0. read base package.json
    // 1. read all sub module packages
    // 2. include sub module versions in modules hash
    // if options.local === true, resolve local dependencies as file paths: "module-a": "../module-a"
    // in general you need non relative dependencies only before publishing
    const localDependencies = tsmOptions.reduce((memo, val) => {
        memo[val.pkg.name] = options.local === false ? val.pkg.version : path.resolve(val.dist);
        return memo;
    }, {});
    // 3. merge packages
    return Promise.all(tsmOptions.map(optPkg => buildPkgJson(optPkg, localDependencies, options)));
    // 4. validate required fields in packages
    // todo:
}
exports.buildPkgs = buildPkgs;
