"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chokidar = require("chokidar");
/**
 * Runs a set of tasks for the given project watching if needed
 * @param project The project location
 * @param tasks A list of tasks to run (listr)
 * @param watch If we should watch or not
 * @returns {Promise<void>}
 */
function tasksWatch({ project, tasks, watch }) {
    // Set tasks runnign to false as none should be
    let isRunning = false;
    // Now run our tasks
    runTasks();
    // If we need to watch then watch for event and run tasks
    if (watch) {
        chokidar.watch(project, { ignored: /[\/\\]\./ })
            .on('change', (event) => {
            console.log(`Changes detected: ${event}`);
            runTasks();
        });
    }
    // Return a promise
    return Promise.resolve();
    /**
     * runs listr tasks
     * @returns {Promise<T>|Promise<TResult|T>|Promise<U>}
     */
    function runTasks() {
        // Check if we already have tasks running if so the return
        if (isRunning) {
            return;
        }
        // Show we are starting the tasks
        console.log(`\n------S-T-A-R-T-I-N-G--T-A-S-K------\n`);
        // Set is running to true as tasks are now started
        isRunning = true;
        // Return the running tasks and signal on completion else
        // Catch any errors
        return tasks.run()
            .then(() => {
            console.log(`\n------C-O-M-P-L-E-T-E-D--T-A-S-K------\n`);
            isRunning = false; // Rest the task running to false
        })
            .catch(err => {
            console.error(`\n${err.message}`);
            isRunning = false; // Rest the task running to false
        });
    }
}
exports.tasksWatch = tasksWatch;
